/* =============================================================================
 *
 * CapSense.c
 *
 * This file contains just the CapSense task.
 *
 * v1.0	YFS	Initial release. 
 *
 * =============================================================================
*/

/*
* Copyright (2014), Cypress Semiconductor Corporation. All Rights Reserved.
*
* This software is owned by Cypress Semiconductor Corporation (Cypress)
* and is protected by and subject to worldwide patent protection (United
* States and foreign), United States copyright laws and international treaty
* provisions. Cypress hereby grants to licensee a personal, non-exclusive,
* non-transferable license to copy, use, modify, create derivative works of,
* and compile the Cypress Source Code and derivative works for the sole
* purpose of creating custom software in support of licensee product to be
* used only in conjunction with a Cypress integrated circuit as specified in
* the applicable agreement. Any reproduction, modification, translation,
* compilation, or representation of this software except as specified above 
* is prohibited without the express written permission of Cypress.
*
* Disclaimer: CYPRESS MAKES NO WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, WITH 
* REGARD TO THIS MATERIAL, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED 
* WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
* Cypress reserves the right to make changes without further notice to the 
* materials described herein. Cypress does not assume any liability arising out 
* of the application or use of any product or circuit described herein. Cypress 
* does not authorize its products for use as critical components in life-support 
* systems where a malfunction or failure may reasonably be expected to result in 
* significant injury to the user. The inclusion of Cypress' product in a life-
* support systems application implies that the manufacturer assumes all risk of 
* such use and in doing so indemnifies Cypress against all charges. 
*
* Use of this Software may be limited by and subject to the applicable Cypress
* software license agreement. 
*/

/* Include FreeRTOS APIs and defines */
#include "FreeRTOS.h"
#include "task.h"
#include "queue.h"
#include "event_groups.h"

/* Include PSoC system and component APIs and defines */
#include <project.h>

/* Include application function declarations and defines */
#include <Task_Defs.h>
#include <OS_Resources.h>
  
/* Define the queue for Capsense data */
QueueHandle_t CapSense_Q;

/* No activity result, returned from CapSense component APIs */
#define NO_FINGER 0xFFFF

/*
 * Function:	CapSense_Task
 *
 * Post CapSense events to a mailbox.
 *
 *              Start CapSense
 *           -> Begin a scan and wait for a result
 *          |   If result is valid (finger) and changed, send it to queue
 *           - Delay to let other tasks run
 *
 * Globals:     CapSense_Q and Button_Events
 *
 * Parameters:	pvParameters - not used
 *
 * Return:		None (never returns)
 */
void CapSense_Task( void *pvParameters )
{
    uint16 curPos = NO_FINGER;              /* Current position */
    uint16 oldPos = NO_FINGER;              /* Last position - prevent repeat posts */
    
    /* Start the CapSense component and initialize baselines */
    CapSense_Start();	
    CapSense_InitializeAllBaselines();
    
    for( ; ; )
    {
        /* Update all baselines and start scanning enabled sensors */
        CapSense_UpdateEnabledBaselines();
    	CapSense_ScanEnabledWidgets();
    
		while( CapSense_IsBusy() != 0 )     /* Wait for scanning to complete */
        {
            vTaskDelay( ONE_MILLISECOND );  /* 1ms delay should suffice */
        }
        
        /* Read the result from the CapSense block */
        curPos = CapSense_GetCentroidPos(CapSense_LINEARSLIDER0__LS);

        /* If there was a finger present and it moved, post the message */
        if( ( curPos != NO_FINGER ) && ( curPos != oldPos ) )
        {
            xQueueSend(
                CapSense_Q,
                (void *)&curPos,
                DO_NOT_WAIT );
            
            oldPos = curPos;                /* Remember the last result */
        }
        
        vTaskDelay( TASK_UPDATE_RATE );     /* Do not scan constantly   */
        
    } /* for( ; ; ) */
    
} /* void CapSense_Task( void ) */
